MODULE geo2ocean
   !!======================================================================
   !!                     ***  MODULE  geo2ocean  ***
   !! Ocean mesh    :  ???
   !!======================================================================
   !! History :  OPA  ! 1989-07  (O. Marti)  Original code
   !!   NEMO     1.0  ! 2002-08  (G. Madec)  F90: Free form + opt.
   !!             -   ! 2006-02  (A. Caubel)  oce2geo - Original code
   !!            2.0  ! 2007-04  (S. Masson)  Add T, F points and bugfix in cos lateral boundary
   !!            3.0  ! 2008-07  (G. Madec)  geo2oce suppress lon/lat agruments
   !!            3.3  ! 2010-10  (K. Mogensen) add obs_rot
   !!            4.0  ! 2011-02  (G. Madec)  dynamical allocation + addition of angle_geo
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   angle_msh_geo    : local sin/cos of the angle between model grid lines and the North direction
   !!   angle_geo        : local sin/cos of the latitude and longitude at each mesh grid point
   !!   geo2oce, oce2geo : ?
   !!   obs_rot          : provide to OBS operator the sin &cos at u- and v-points
   !!
   !!   repcmo, repere, rot_rep :   old routines  ==> to be suppressed
   !!----------------------------------------------------------------------
   USE dom_oce        ! mesh and scale factors
   USE phycst         ! physical constants
   USE in_out_manager ! I/O manager
   USE lbclnk         ! ocean lateral boundary conditions (or mpp link)

   IMPLICIT NONE
   PRIVATE

   PUBLIC   geo2oce, oce2geo   !    
   PUBLIC   obs_rot            !
   !
   PUBLIC   repcmo, repere, rot_rep    ! CAUTION: these routines are kept only for compatibility.
   !                                   !          They are only a useless overlay of rot_rep

   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:) ::   gsin, gcos         ! sinus & cosinus at T,U,V,F points
   !                                                                     ! between mesh and NP direction
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:) ::   gsinlon, gsinlat   ! sinus   of lon & lat at T,U,V,F points
   REAL(wp), ALLOCATABLE, SAVE, DIMENSION(:,:,:) ::   gcoslon, gcoslat   ! cosinus of lon & lat at T,U,V,F points

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 4.0 , NEMO Consortium (2011)
   !! $Id$ 
   !! Software governed by the CeCILL licence     (NEMOGCM/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------
CONTAINS

   SUBROUTINE angle_msh_geo
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE angle_msh_geo  ***
      !! 
      !! ** Purpose :   Compute angles between model grid lines and the North direction
      !!
      !! ** Action  :   allocate and compute (gsint, gcost, gsinu, gcosu, gsinv, 
      !!              gcosv, gsinf, gcosf) arrays: sinus and cosinus of the angle 
      !!              between the north-south axe and the j-direction at t, u, v and f-points
      !!----------------------------------------------------------------------
      INTEGER  ::   ji, jj   ! dummy loop indices
      INTEGER  ::   ierr     ! local integer
      REAL(wp) ::   zpi_4    ! local scalar (pi/4)
      REAL(wp) ::   zlam, zphi, zlan, zphh   ! local scalars
      REAL(wp) ::   zxnpt, zynpt, znnpt      ! x,y components & norm of the vector: T point to North Pole
      REAL(wp) ::   zxnpu, zynpu, znnpu      ! x,y components & norm of the vector: U point to North Pole
      REAL(wp) ::   zxnpv, zynpv, znnpv      ! x,y components & norm of the vector: V point to North Pole
      REAL(wp) ::   zxnpf, zynpf, znnpf      ! x,y components & norm of the vector: F point to North Pole
      REAL(wp) ::   zxvvt, zyvvt, znvvt      ! x,y components & norm of the vector: between V pts below and above a T pt
      REAL(wp) ::   zxffu, zyffu, znffu      ! x,y components & norm of the vector: between F pts below and above a U pt
      REAL(wp) ::   zxffv, zyffv, znffv      ! x,y components & norm of the vector: between F pts left  and right a V pt
      REAL(wp) ::   zxuuf, zyuuf, znuuf      ! x,y components & norm of the vector: between U pts below and above a F pt
      !!----------------------------------------------------------------------
      !
      ! already allocated & initialized  ==> return
      ! -------------------------------
      IF( ALLOCATED( gsin ) .AND. ALLOCATED( gcos )  )   RETURN

      ! allocate cos & sin arrays 
      ! -------------------------
      ALLOCATE( gsin(jpi,jpj,4) , gcos(jpi,jpj,4) , STAT=ierr )
      IF(lk_mpp)   CALL mpp_sum( ierr )
      IF( ierr /= 0 )   CALL ctl_stop('STOP', 'angle_msh_geo: unable to allocate arrays' )
      !
      ! initialize cos & sin arrays
      ! ----------------------------
      ! (computation done on the north stereographic polar plane)
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*) ' angle_msh_geo : geographic <--> model mesh , cos/sin initialization '
      IF(lwp) WRITE(numout,*) ' ~~~~~~~~~~~~~~'

      zpi_4 = rpi / 4._wp
      !
      DO jj = 2, jpjm1
!CDIR NOVERRCHK
         DO ji = fs_2, jpi   ! vector opt.
            !
            zlam = glamt(ji,jj)            ! north pole direction & modulous (at t-point)
            zphi = gphit(ji,jj)
            zxnpt = 0._wp - 2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            zynpt = 0._wp - 2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            znnpt = zxnpt*zxnpt + zynpt*zynpt

            zlam = glamu(ji,jj)            ! north pole direction & modulous (at u-point)
            zphi = gphiu(ji,jj)
            zxnpu = 0._wp - 2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            zynpu = 0._wp - 2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            znnpu = zxnpu*zxnpu + zynpu*zynpu

            zlam = glamv(ji,jj)            ! north pole direction & modulous (at v-point)
            zphi = gphiv(ji,jj)
            zxnpv = 0._wp - 2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            zynpv = 0._wp - 2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            znnpv = zxnpv*zxnpv + zynpv*zynpv
            
            zlam = glamf(ji,jj)            ! north pole direction & modulous (at f-point)
            zphi = gphif(ji,jj)
            zxnpf = 0._wp - 2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            zynpf = 0._wp - 2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )
            znnpf = zxnpf*zxnpf + zynpf*zynpf

            zlam = glamv(ji,jj  )            ! j-direction: v-point segment direction (around t-point)
            zphi = gphiv(ji,jj  )
            zlan = glamv(ji,jj-1)
            zphh = gphiv(ji,jj-1)
            zxvvt =  2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * COS( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            zyvvt =  2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * SIN( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            znvvt = SQRT( znnpt * ( zxvvt*zxvvt + zyvvt*zyvvt )  )
            znvvt = MAX( znvvt, 1.e-14 )

            zlam = glamf(ji,jj  )            ! j-direction: f-point segment direction (around u-point)
            zphi = gphif(ji,jj  )
            zlan = glamf(ji,jj-1)
            zphh = gphif(ji,jj-1)
            zxffu =  2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * COS( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            zyffu =  2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * SIN( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            znffu = SQRT( znnpu * ( zxffu*zxffu + zyffu*zyffu )  )
            znffu = MAX( znffu, 1.e-14 )

            zlam = glamf(ji  ,jj)            ! i-direction: f-point segment direction (around v-point)
            zphi = gphif(ji  ,jj)
            zlan = glamf(ji-1,jj)
            zphh = gphif(ji-1,jj)
            zxffv =  2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * COS( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            zyffv =  2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * SIN( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            znffv = SQRT( znnpv * ( zxffv*zxffv + zyffv*zyffv )  )
            znffv = MAX( znffv, 1.e-14 )

            zlam = glamu(ji,jj+1)            ! j-direction: u-point segment direction (around f-point)
            zphi = gphiu(ji,jj+1)
            zlan = glamu(ji,jj  )
            zphh = gphiu(ji,jj  )
            zxuuf =  2._wp * COS( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * COS( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            zyuuf =  2._wp * SIN( rad*zlam ) * TAN( zpi_4 - rad*zphi*0.5 )   &
               &  -  2._wp * SIN( rad*zlan ) * TAN( zpi_4 - rad*zphh*0.5 )
            znuuf = SQRT( znnpf * ( zxuuf*zxuuf + zyuuf*zyuuf )  )
            znuuf = MAX( znuuf, 1.e-14 )

            ! cosinus and sinus using scalar and vectorial products
            gsin(ji,jj,1) = ( zxnpt*zyvvt - zynpt*zxvvt ) / znvvt      ! T-point
            gcos(ji,jj,1) = ( zxnpt*zxvvt + zynpt*zyvvt ) / znvvt

            gsin(ji,jj,2) = ( zxnpu*zyffu - zynpu*zxffu ) / znffu      ! U-point
            gcos(ji,jj,2) = ( zxnpu*zxffu + zynpu*zyffu ) / znffu
            !
            gsin(ji,jj,3) = ( zxnpv*zxffv + zynpv*zyffv ) / znffv        ! F-point
            gcos(ji,jj,3) =-( zxnpv*zyffv - zynpv*zxffv ) / znffv            ! (caution, rotation of 90 degres)
            !
            gsin(ji,jj,4) = ( zxnpf*zyuuf - zynpf*zxuuf ) / znuuf      ! V-point
            gcos(ji,jj,4) = ( zxnpf*zxuuf + zynpf*zyuuf ) / znuuf
            !
         END DO
      END DO
      !
      ! Geographic mesh case
      ! --------------------
      DO jj = 2, jpjm1
         DO ji = fs_2, jpi   ! vector opt.
            IF( MOD( ABS( glamv(ji,jj) - glamv(ji,jj-1) ), 360. ) < 1.e-8 ) THEN
               gsin(ji,jj,1) = 0.
               gcos(ji,jj,1) = 1.
            ENDIF
            IF( MOD( ABS( glamf(ji,jj) - glamf(ji,jj-1) ), 360. ) < 1.e-8 ) THEN
               gsin(ji,jj,2) = 0.
               gcos(ji,jj,2) = 1.
            ENDIF
            IF(      ABS( gphif(ji,jj) - gphif(ji-1,jj) )         < 1.e-8 ) THEN
               gsin(ji,jj,3) = 0.
               gcos(ji,jj,3) = 1.
            ENDIF
            IF( MOD( ABS( glamu(ji,jj) - glamu(ji,jj+1) ), 360. ) < 1.e-8 ) THEN
               gsin(ji,jj,4) = 0.
               gcos(ji,jj,4) = 1.
            ENDIF
         END DO
      END DO
      !
      CALL lbc_lnk( gcos(:,:,1), 'T', -1. )   ;   CALL lbc_lnk( gsin(:,:,1), 'T', -1. )      ! lateral boundary cond.
      CALL lbc_lnk( gcos(:,:,2), 'U', -1. )   ;   CALL lbc_lnk( gsin(:,:,2), 'U', -1. )
      CALL lbc_lnk( gcos(:,:,3), 'V', -1. )   ;   CALL lbc_lnk( gsin(:,:,3), 'V', -1. )
      CALL lbc_lnk( gcos(:,:,4), 'F', -1. )   ;   CALL lbc_lnk( gsin(:,:,4), 'F', -1. )
      !
   END SUBROUTINE angle_msh_geo


   SUBROUTINE angle_geo
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE angle_geo  ***
      !!      
      !! ** Purpose :   compute one for all, and at each mesh grid points, 
      !!              the local cos/sin of latitude/longitude.
      !!----------------------------------------------------------------------
      INTEGER ierr
      !!----------------------------------------------------------------------
      !
      IF( ALLOCATED( gsinlon ) .AND. ALLOCATED( gcoslon ) .AND.   &   !==  already allocated & initialized
          ALLOCATED( gsinlat ) .AND. ALLOCATED( gcoslat )         )   RETURN

      ALLOCATE( gsinlon(jpi,jpj,4) , gcoslon(jpi,jpj,4) ,     &       !==  allocate the arrays
         &      gsinlat(jpi,jpj,4) , gcoslat(jpi,jpj,4) , STAT=ierr )
      IF( lk_mpp    )   CALL mpp_sum( ierr )
      IF( ierr /= 0 )   CALL ctl_stop( 'STOP', 'angle_geo: unable to allocate arrays')
         !
      gsinlon(:,:,1) = SIN( rad * glamt(:,:) )     ! T-point
      gcoslon(:,:,1) = COS( rad * glamt(:,:) )
      gsinlat(:,:,1) = SIN( rad * gphit(:,:) )
      gcoslat(:,:,1) = COS( rad * gphit(:,:) )
      !
      gsinlon(:,:,2) = SIN( rad * glamu(:,:) )     ! U-point
      gcoslon(:,:,2) = COS( rad * glamu(:,:) )
      gsinlat(:,:,2) = SIN( rad * gphiu(:,:) )
      gcoslat(:,:,2) = COS( rad * gphiu(:,:) )
      !
      gsinlon(:,:,3) = SIN( rad * glamv(:,:) )     ! V-point
      gcoslon(:,:,3) = COS( rad * glamv(:,:) )
      gsinlat(:,:,3) = SIN( rad * gphiv(:,:) )
      gcoslat(:,:,3) = COS( rad * gphiv(:,:) )
      !
      gsinlon(:,:,4) = SIN( rad * glamf(:,:) )     ! T-point
      gcoslon(:,:,4) = COS( rad * glamf(:,:) )
      gsinlat(:,:,4) = SIN( rad * gphif(:,:) )
      gcoslat(:,:,4) = COS( rad * gphif(:,:) )
      !
   END SUBROUTINE angle_geo


   SUBROUTINE geo2oce( pxx, pyy, pzz, cgrid, pte, ptn )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE geo2oce  ***
      !!      
      !! ** Purpose :   Change a vector from geocentric to east/north
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   pxx, pyy, pzz   !
      CHARACTER(len=1)            , INTENT(in   ) ::   cgrid           ! type of grid point
      REAL(wp), DIMENSION(jpi,jpj), INTENT(  out) ::   pte, ptn        !
      !!
      INTEGER ::   ipt     ! local integer
      !!----------------------------------------------------------------------

      CALL angle_geo             ! initialization of coc & sin (just a return if not the 1st call)
      !
      SELECT CASE (cgrid)        ! type of point
      CASE ('T')   ;   ipt = 1
      CASE ('U')   ;   ipt = 2
      CASE ('V')   ;   ipt = 3
      CASE ('F')   ;   ipt = 4
      CASE DEFAULT   ;   CALL ctl_stop( 'Only T, U, V and F grid points are coded' )
      END SELECT
  
      !                          ! transformation
      pte(:,:) = - gsinlon(:,:,ipt)                    * pxx(:,:) + gcoslon(:,:,ipt) * pyy(:,:)
      ptn(:,:) = - gsinlat(:,:,ipt) * gsinlat(:,:,ipt) * pxx(:,:)    &
         &       - gsinlon(:,:,ipt) * gsinlat(:,:,ipt) * pyy(:,:) + gcoslat(:,:,ipt) * pzz(:,:)
!!$   ptv(:,:) =   gsinlat(:,:,ipt) * gcoslat(:,:,ipt) * pxx(:,:)    &
!!$              + zsinlon(:,:,ipt) * gcoslat(:,:,ipt) * pyy(:,:) + gsinlat(:,:,ipt) * pzz(:,:)
      !
   END SUBROUTINE geo2oce


   SUBROUTINE oce2geo ( pte, ptn, cgrid, pxx , pyy , pzz )
      !!----------------------------------------------------------------------
      !!                    ***  ROUTINE oce2geo  ***
      !!      
      !! ** Purpose :   Change vector from east/north to geocentric
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:,:), INTENT(in   ) ::  pte, ptn          ! 
      CHARACTER(len=1)        , INTENT(in   ) ::  cgrid             !
      REAL(wp), DIMENSION(:,:), INTENT(  out) ::  pxx , pyy , pzz   !
      !!
      INTEGER ::   ipt     !
      !!----------------------------------------------------------------------
      !
      CALL angle_geo             ! initialization of coc & sin (just a return if not the 1st call)
      !
      SELECT CASE (cgrid)        ! type of point
      CASE ('T')   ;   ipt = 1
      CASE ('U')   ;   ipt = 2
      CASE ('V')   ;   ipt = 3
      CASE ('F')   ;   ipt = 4
      CASE DEFAULT   ;   CALL ctl_stop( 'Only T, U, V and F grid points are coded' )
      END SELECT
      !
      !                          ! transformation
      pxx(:,:) = - gsinlon(:,:,ipt) * pte(:,:) - gcoslon(:,:,ipt) * gsinlat(:,:,ipt) * ptn(:,:) 
      pyy(:,:) =   gcoslon(:,:,ipt) * pte(:,:) - gsinlon(:,:,ipt) * gsinlat(:,:,ipt) * ptn(:,:)
      pzz(:,:) =   gcoslat(:,:,ipt) * ptn(:,:)
      !
   END SUBROUTINE oce2geo


   SUBROUTINE obs_rot( psinu, pcosu, psinv, pcosv )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE obs_rot  ***
      !!
      !! ** Purpose :   provide to OBS operator the sin &cos at u- and v-points
      !!              in order to rotate currents at observation points
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(out)::   psinu, pcosu, psinv, pcosv   ! copy of data
      !!----------------------------------------------------------------------
      !
      CALL angle_msh_geo      ! initialization of coc & sin (just a return if not the 1st call)
      !
      psinu(:,:) = gsin(:,:,2)      ! U-point
      pcosu(:,:) = gcos(:,:,2)
      psinv(:,:) = gsin(:,:,3)      ! V-point
      pcosv(:,:) = gcos(:,:,3)
      !
   END SUBROUTINE obs_rot


   SUBROUTINE repcmo( pxu1, pyu1, pxv1, pyv1, px2 , py2 )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE repcmo  ***
      !!
      !! ** Purpose :   Change vector componantes from a geographic grid to a
      !!              stretched coordinates grid.
      !!
      !! ** Method  :   Initialization of arrays at the first call.
      !!
      !! ** Action  : - px2 : first  componante (defined at u point)
      !!              - py2 : second componante (defined at v point)
      !!----------------------------------------------------------------------
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   pxu1, pyu1   ! geographic vector componantes at u-point
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj) ::   pxv1, pyv1   ! geographic vector componantes at v-point
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   px2          ! i-componante (defined at u-point)
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj) ::   py2          ! j-componante (defined at v-point)
      !!----------------------------------------------------------------------
      !      
      CALL rot_rep( pxu1, pyu1, 'U', 'en->i',px2 )    ! Change from geographic to stretched coordinate
      CALL rot_rep( pxv1, pyv1, 'V', 'en->j',py2 )
      !
   END SUBROUTINE repcmo


   SUBROUTINE repere( px1, py1, px2, py2, kchoix, cd_type )
      !!----------------------------------------------------------------------
      !!                 ***  ROUTINE repere  ***
      !!        
      !! ** Purpose :   Change vector componantes between a geopgraphic grid 
      !!              and a stretched coordinates grid.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(:,:)  , INTENT(in   ) ::   px1, py1   ! the 2 horizontal components to be rotated
      REAL(wp), DIMENSION(:,:)  , INTENT(  out) ::   px2, py2   ! the 2 horizontal components in the model repere
      INTEGER                   , INTENT(in   ) ::   kchoix     ! = 1 change from geographic to model grid.
      !                                                         ! =-1 change from model to geographic grid
      CHARACTER(len=1), OPTIONAL, INTENT(in   ) ::   cd_type    ! define the nature of pt2d array grid-points
      !
      CHARACTER(len=1) ::   cl_type      ! define the nature of pt2d array grid-points (T point by default)
      !!----------------------------------------------------------------------
      !
      cl_type = 'T'
      IF( PRESENT(cd_type) )   cl_type = cd_type
         !
      SELECT CASE (kchoix)
      CASE ( 1)      ! change from geographic to model grid.
         CALL rot_rep( px1, py1, cl_type, 'en->i', px2 )
         CALL rot_rep( px1, py1, cl_type, 'en->j', py2 )
      CASE (-1)      ! change from model to geographic grid
         CALL rot_rep( px1, py1, cl_type, 'ij->e', px2 )
         CALL rot_rep( px1, py1, cl_type, 'ij->n', py2 )
      CASE DEFAULT   ;   CALL ctl_stop( 'repere: Syntax Error in the definition of kchoix (1 OR -1' )
      END SELECT
      !
   END SUBROUTINE repere


   SUBROUTINE rot_rep( pxin, pyin, cd_type, cdtodo, prot )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE rot_rep  ***
      !!
      !! ** Purpose :   Rotate the Repere: Change vector componantes between
      !!                geographic grid <--> stretched coordinates grid.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(in   ) ::   pxin, pyin   ! vector componantes
      CHARACTER(len=1),             INTENT(in   ) ::   cd_type      ! define the nature of pt2d array grid-points
      CHARACTER(len=5),             INTENT(in   ) ::   cdtodo       ! specify the work to do:
      !!                                                            ! 'en->i' east-north componantes to model i-componante
      !!                                                            ! 'en->j' east-north componantes to model j-componante
      !!                                                            ! 'ij->e' model i-j componantes to east componante
      !!                                                            ! 'ij->n' model i-j componantes to east componante
      REAL(wp), DIMENSION(jpi,jpj), INTENT(  out) ::   prot         !
      !
      INTEGER ::   ipt   !
      !!----------------------------------------------------------------------

      CALL angle_msh_geo   ! initialization of the transformation (just a return if not the 1st call)
      
      SELECT CASE (cd_type)
      CASE ('T')   ;   ipt = 1
      CASE ('U')   ;   ipt = 2
      CASE ('V')   ;   ipt = 3
      CASE ('F')   ;   ipt = 4
      CASE DEFAULT   ;   CALL ctl_stop( 'Only T, U, V and F grid points are coded' )
      END SELECT
      
      SELECT CASE (cdtodo)
      CASE ('en->i')   ;   prot(:,:) = pxin(:,:) * gcos(:,:,ipt) + pyin(:,:) * gsin(:,:,ipt)    ! east-north to model i-component
      CASE ('en->j')   ;   prot(:,:) = pyin(:,:) * gcos(:,:,ipt) - pxin(:,:) * gsin(:,:,ipt)    ! east-north to model j-component
      CASE ('ij->e')   ;   prot(:,:) = pxin(:,:) * gcos(:,:,ipt) - pyin(:,:) * gsin(:,:,ipt)    ! model i-j  to east    component
      CASE ('ij->n')   ;   prot(:,:) = pyin(:,:) * gcos(:,:,ipt) + pxin(:,:) * gsin(:,:,ipt)    ! model i-j  to north   component
      CASE DEFAULT   ;   CALL ctl_stop( 'rot_rep: Syntax Error in the definition of cdtodo' )
      END SELECT
      
   END SUBROUTINE rot_rep

  !!======================================================================
END MODULE geo2ocean
